@@ -33,6 +33,8 @@ export class OperatorTasks {
    * Returns tasks list which perform preflight platform checks.
    */
   startTasks(flags: any, command: Command): Listr {
+    const clusterRoleName = `${flags.chenamespace}-${this.operatorClusterRole}`
+    const clusterRoleBindingName = `${flags.chenamespace}-${this.operatorClusterRoleBinding}`
     const kube = new KubeHelper(flags)
     if (isStableVersion(flags)) {
       command.warn('Consider using the more reliable \'OLM\' installer when deploying a stable release of CodeReady Workspaces (--installer=olm).')
@@ -70,14 +72,14 @@ export class OperatorTasks {
         }
       },
       {
-        title: `Create ClusterRole ${this.operatorClusterRole}`,
+        title: `Create ClusterRole ${clusterRoleName}`,
         task: async (ctx: any, task: any) => {
-          const exist = await kube.clusterRoleExist(this.operatorClusterRole)
+          const exist = await kube.clusterRoleExist(clusterRoleName)
           if (exist) {
             task.title = `${task.title}...It already exists.`
           } else {
             const yamlFilePath = ctx.resourcesPath + 'cluster_role.yaml'
-            const statusCode = await kube.createClusterRoleFromFile(yamlFilePath)
+            const statusCode = await kube.createClusterRoleFromFile(yamlFilePath, clusterRoleName)
             if (statusCode === 403) {
               command.error('ERROR: It looks like you don\'t have enough privileges. You need to grant more privileges to current user or use a different user. If you are using minishift you can "oc login -u system:admin"')
             }
@@ -99,13 +101,13 @@ export class OperatorTasks {
         }
       },
       {
-        title: `Create ClusterRoleBinding ${this.operatorClusterRoleBinding}`,
+        title: `Create ClusterRoleBinding ${clusterRoleBindingName}`,
         task: async (_ctx: any, task: any) => {
-          const exist = await kube.clusterRoleBindingExist(this.operatorRoleBinding)
+          const exist = await kube.clusterRoleBindingExist(clusterRoleBindingName)
           if (exist) {
             task.title = `${task.title}...It already exists.`
           } else {
-            await kube.createClusterRoleBinding(this.operatorClusterRoleBinding, this.operatorServiceAccount, flags.chenamespace, this.operatorClusterRole)
+            await kube.createClusterRoleBinding(clusterRoleBindingName, this.operatorServiceAccount, flags.chenamespace, clusterRoleName)
             task.title = `${task.title}...done.`
           }
         }
@@ -114,10 +116,15 @@ export class OperatorTasks {
         title: `Create CRD ${this.cheClusterCrd}`,
         task: async (ctx: any, task: any) => {
           const exist = await kube.crdExist(this.cheClusterCrd)
+          const yamlFilePath = ctx.resourcesPath + 'crds/org_v1_che_crd.yaml'
+
           if (exist) {
+            const checkCRD = await kube.isCRDCompatible(this.cheClusterCrd, yamlFilePath)
+            if (!checkCRD) {
+              cli.error(`It is not possible to proceed the installation of CodeReady Workspaces. The existed ${this.cheClusterCrd} is different from a new one. Please update it to continue the installation.`)
+            }
             task.title = `${task.title}...It already exists.`
           } else {
-            const yamlFilePath = ctx.resourcesPath + 'crds/org_v1_che_crd.yaml'
             await kube.createCrdFromFile(yamlFilePath)
             task.title = `${task.title}...done.`
           }
@@ -171,6 +178,8 @@ export class OperatorTasks {
 
   updateTasks(flags: any, command: Command): Listr {
     const kube = new KubeHelper(flags)
+    const clusterRoleName = `${flags.chenamespace}-${this.operatorClusterRole}`
+    const clusterRoleBindingName = `${flags.chenamespace}-${this.operatorClusterRoleBinding}`
     return new Listr([
       copyOperatorResources(flags, command.config.cacheDir),
       {
@@ -208,18 +217,26 @@ export class OperatorTasks {
         }
       },
       {
-        title: `Updating ClusterRole ${this.operatorClusterRole}`,
+        title: `Updating ClusterRole ${clusterRoleName}`,
         task: async (ctx: any, task: any) => {
-          const exist = await kube.clusterRoleExist(this.operatorClusterRole)
+          const clusterRoleExists = await kube.clusterRoleExist(clusterRoleName)
+          const legacyClusterRoleExists = await kube.clusterRoleExist(this.operatorClusterRole)
           const yamlFilePath = ctx.resourcesPath + 'cluster_role.yaml'
-          if (exist) {
-            const statusCode = await kube.replaceClusterRoleFromFile(yamlFilePath)
+          if (clusterRoleExists) {
+            const statusCode = await kube.replaceClusterRoleFromFile(yamlFilePath, clusterRoleName)
             if (statusCode === 403) {
               command.error('ERROR: It looks like you don\'t have enough privileges. You need to grant more privileges to current user or use a different user. If you are using minishift you can "oc login -u system:admin"')
             }
             task.title = `${task.title}...updated.`
+          // it is needed to check the legacy cluster object name to be compatible with previous installations
+          } else if (legacyClusterRoleExists) {
+            const statusCode = await kube.replaceClusterRoleFromFile(yamlFilePath, this.operatorClusterRole)
+            if (statusCode === 403) {
+              command.error('ERROR: It looks like you don\'t have enough privileges. You need to grant more privileges to current user or use a different user. If you are using minishift you can "oc login -u system:admin"')
+            }
+            task.title = `Updating ClusterRole ${this.operatorClusterRole}...updated.`
           } else {
-            const statusCode = await kube.createClusterRoleFromFile(yamlFilePath)
+            const statusCode = await kube.createClusterRoleFromFile(yamlFilePath, clusterRoleName)
             if (statusCode === 403) {
               command.error('ERROR: It looks like you don\'t have enough privileges. You need to grant more privileges to current user or use a different user. If you are using minishift you can "oc login -u system:admin"')
             }
@@ -242,14 +259,19 @@ export class OperatorTasks {
         }
       },
       {
-        title: `Updating ClusterRoleBinding ${this.operatorClusterRoleBinding}`,
+        title: `Updating ClusterRoleBinding ${clusterRoleBindingName}`,
         task: async (_ctx: any, task: any) => {
-          const exist = await kube.clusterRoleBindingExist(this.operatorRoleBinding)
-          if (exist) {
-            await kube.replaceClusterRoleBinding(this.operatorClusterRoleBinding, this.operatorServiceAccount, flags.chenamespace, this.operatorClusterRole)
+          const clusterRoleBindExists = await kube.clusterRoleBindingExist(clusterRoleBindingName)
+          const legacyClusterRoleBindExists = await kube.clusterRoleBindingExist(this.operatorClusterRoleBinding)
+          if (clusterRoleBindExists) {
+            await kube.replaceClusterRoleBinding(clusterRoleBindingName, this.operatorServiceAccount, flags.chenamespace, clusterRoleName)
             task.title = `${task.title}...updated.`
+          // it is needed to check the legacy cluster object name to be compatible with previous installations
+          } else if (legacyClusterRoleBindExists) {
+            await kube.replaceClusterRoleBinding(this.operatorClusterRoleBinding, this.operatorServiceAccount, flags.chenamespace, this.operatorClusterRole)
+            task.title = `Updating ClusterRoleBinding ${this.operatorClusterRoleBinding}...updated.`
           } else {
-            await kube.createClusterRoleBinding(this.operatorClusterRoleBinding, this.operatorServiceAccount, flags.chenamespace, this.operatorClusterRole)
+            await kube.createClusterRoleBinding(clusterRoleBindingName, this.operatorServiceAccount, flags.chenamespace, clusterRoleName)
             task.title = `${task.title}...created new one.`
           }
         }
@@ -307,6 +329,8 @@ export class OperatorTasks {
    */
   deleteTasks(flags: any): ReadonlyArray<Listr.ListrTask> {
     let kh = new KubeHelper(flags)
+    const clusterRoleName = `${flags.chenamespace}-${this.operatorClusterRole}`
+    const clusterRoleBindingName = `${flags.chenamespace}-${this.operatorClusterRoleBinding}`
     return [{
       title: `Delete the Custom Resource of type ${CHE_CLUSTER_CRD}`,
       task: async (_ctx: any, task: any) => {
@@ -334,21 +358,33 @@ export class OperatorTasks {
       }
     },
     {
-      title: `Delete cluster role binding ${this.operatorClusterRoleBinding}`,
+      title: `Delete cluster role binding ${clusterRoleBindingName}`,
       task: async (_ctx: any, task: any) => {
-        if (await kh.clusterRoleBindingExist(this.operatorClusterRoleBinding)) {
+        const clusterRoleBindExists = await kh.clusterRoleBindingExist(clusterRoleBindingName)
+        const legacyClusterRoleBindExists = await kh.clusterRoleBindingExist(this.operatorClusterRoleBinding)
+        if (clusterRoleBindExists) {
+          await kh.deleteClusterRoleBinding(clusterRoleBindingName)
+          task.title = await `${task.title}...OK`
+        // it is needed to check the legacy cluster object name to be compatible with previous installations
+        } else if (legacyClusterRoleBindExists) {
           await kh.deleteClusterRoleBinding(this.operatorClusterRoleBinding)
+          task.title = await `Delete cluster role binding ${this.operatorClusterRoleBinding}...OK`
         }
-        task.title = await `${task.title}...OK`
       }
     },
     {
-      title: `Delete cluster role ${this.operatorClusterRole}`,
+      title: `Delete cluster role ${clusterRoleName}`,
       task: async (_ctx: any, task: any) => {
-        if (await kh.clusterRoleExist(this.operatorClusterRole)) {
+        const clusterRoleExists = await kh.clusterRoleExist(clusterRoleName)
+        const legacyClusterRoleExists = await kh.clusterRoleExist(this.operatorClusterRole)
+        if (clusterRoleExists) {
+          await kh.deleteClusterRole(clusterRoleName)
+          task.title = await `${task.title}...OK`
+        // it is needed to check the legacy cluster object name to be compatible with previous installations
+        } else if (legacyClusterRoleExists) {
           await kh.deleteClusterRole(this.operatorClusterRole)
+          task.title = await `Delete cluster role ${this.operatorClusterRole}...OK`
         }
-        task.title = await `${task.title}...OK`
       }
     },
     {
